// Polaris (CB Radio) Version 1.1.1  03/03/2023   by F.Lucesio
// Version history
// 1.0  base prese alcune routine del Ciucc box 
// 1.1  modificato rumori sintonia che si avviano sul movimento

#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"



// mp3 palyer settings ------------------
// Use pins 2 and 3 to communicate with DFPlayer Mini
static const uint8_t PIN_MP3_TX = 2; // Connects to module's RX 
static const uint8_t PIN_MP3_RX = 3; // Connects to module's TX 
SoftwareSerial softwareSerial(PIN_MP3_RX, PIN_MP3_TX);

// Create the Player object
DFRobotDFPlayerMini Player;

// PERSONALIZZAZIONE --------------------------------------------------

// costanti

const byte istvol = 30;                       // isteresi sulla lettura analog del volume
const byte istsint = 20;                      // isteresi  sulla lettura analog della sintonia
const byte maxmode = 5;                       // massimo numero di modalita

// variabili di controllo

// vettore contentente il numero di elementi per ogni folder
byte fsize[11] = {0,177,80,83,65,250,96,83,252,60,19}; 
//                   1  2  3  4   5  6  7   8  9  0
// CARTELLA ----------------------- CONTENUTO ---------------------------

// 01: (default) musica rigorosamente anni 60-70 o antecedente               
// 02: musica presa da vecchie registrazioni geloso della radio stessa   
// 03: vecchie trasmissioni di RMC e rai rig: znni 60, 70, tg            
// 04: audio vari personali   anni 60 e 70         
// 05: estratto canzoni di repertorio solo anni fino al 1980                   
// 06: favole e racconti                                               
// 07: letture segrete                                                       
// 08: Aforismi segreti                                                   
// 09: musica classica                                                      
// 10: suoni di sistema (compresi rumori di centratura sintonia)         
// ------------------------------------------------------------------------
int sintpoint[7] = {0,135,249,444,601,767,1023}; // setpoint stazioni sintonia
byte nfold = 10;   // numero complessivo folders
byte nspot = 50;   // numero spot nella cartella ADVERT
int  nafo = 834;   // numero complessivo aforismi normali e oracolo
byte fold[11];     // vettore informazioni cartelle player
byte prog[11][3];  // matrice che contiene il programma caricato
byte foldcor = 1;  // folder corrente 
byte puntcor = 1;  // puntatore al brano del folder corrente
int punt =0;       // puntatore assoluto ai contenuti abilitati
int maxpunt = 0;   // totale contenuti selezionati
int maxfile = 0;   // totale contenuti assoluto (tranne cartella \advert)
int tem=0;         // variabile temporanea

// digital I/O
const byte pin_LED =7;      // led
const byte pinmp3 =5;       // per rilevare attivita' del player: (ON = fermo)
const byte pin_AUDIO =9;    // per rilevare attivita' del player: (ON = fermo)


const byte pinvol = A5;     // potenziometro volume
const byte pinsint = A4;    // potenziometro sintonia


// definizione variabili ----------------------------------------------
// tempo

long int Tempo;          // per ciclo principale
long int Ts;             // per sintonia
long int Tsin;           // per sintonia
long int startplay;      // fissa il tempo di inizio di un item 
long int estempo;        // fissa il tempo di power saving


// temporizzatori (valori in millisecondi)
int Ig = 100;    // ciclo generale 
int Is = 1000;   // per azzeramento sintonizzo
int Isin = 3000; // per cambio audio sintonia


// flags binari

bool aradio = false;      // modo radio on/off :(significa playare in modo continuo senza terminare il pezzo)                    
bool cont = false;        // modo continuo si /no 
bool debug = false;       // spara nel canale seriale delle informazioni utili
bool giafatto = false;    // flag per scrivere una sola volta su EEPROM allo spegnimento  
bool suonato = false;     // per controllare play di spot afo ecc
bool segreto = false;     // abilita contenuto alternativo si/no
bool segnale = false;     // pre uscita segnale sonoro sitonia
bool sintonizzo = false;  // per indicare che sta sintonizzando
bool fondoscala = false;  // indica la sintonia a fondo scala
bool initscala = false;    // indica la sintonia a inizio scala


// varie
byte mode = 1;           // modalita (programma)  
byte rmode = 0;          // per lettura sintonia
byte spot =0;            // ogni quanto va lo spot (0 nulla)
byte afo=0;              // idem per aforismi 
byte seg=0;              // idem per aforismi segreti 
byte cspot =0;            // contatore spot
byte cafo=0;              // contatore aforismi 
byte cseg=0;              // contatore aforismi segreti 

int volume = 0;
int volumep = -1;
int volusint = 0;
int sinto = 0;
int sintop = 0;
int deltas;             // scostamento della sintonia dal setpoint
int deltamax;           // scostamento massimo della sintonia dal setpoint
byte volump3 = 4;       // volume iniziale basso   
byte k,f;



void setup()
{
 // inizializza pins 
 pinMode(pin_LED, OUTPUT);
 pinMode(pin_AUDIO, OUTPUT);
 pinMode(pinvol, INPUT);
 pinMode(pinsint, INPUT);
 pinMode(pinmp3, INPUT);
 
 Serial.begin(9600);
 // Init serial port for DFPlayer Mini
 softwareSerial.begin(9600);
 delay(5000);  // ritardo indispensabile altrimenti il sistema si blocca alla partenza (sempre se alimentato a pile)
 // Start communication with DFPlayer Mini
 if (Player.begin(softwareSerial)) {
 // Set volume (0 to 30)
    Player.volume(volump3);
  } else {
     if(debug) Serial.println("Errore mp3 Player!");
	 while(true){ // loop  led blink = failure
        digitalWrite(pin_LED,HIGH);  
	    delay(200);
        digitalWrite(pin_LED,LOW);  
	    delay(200);
 	}	 
  }
 volume = analogRead(pinvol);
 // setta volume in base al potenziometro la prima volta
 volump3 = map(volume, 0, 1023, 0, 30);
 Player.volume(volump3);

 // Conta numero totale pezzi nelle rispettive dir numeriche
 for(k=1;k<=nfold;k++){
	 maxfile = maxfile + fsize[k];
    }
// trova mode in base alla posizione della sintonia   
sinto = analogRead(pinsint);

mode = dovesono(sinto);
if (mode == 0 || mode >5)
	mode =1;  // setta ev. mode default
leggiprog(mode);
randomSeed(analogRead(pinvol));  // init random 

Tempo=millis();  //inizializza timers
Ts=millis();
Tsin=millis();


}


void loop()
{
//  ciclo principale ----------------------------------------------------------------
if (millis()-Tempo >= Ig) 
{
   Tempo=millis();
   
    // decide cosa mandare in base alle impostazioni programmate
    // manda in play solo se il player è libero e sono passati 
	// almeno 5 secondi dall'avvio dell'ultimo mp3
    if(digitalRead(pinmp3) == HIGH && (millis()-startplay) > 5000  ) {
        // controlla se deve mandare spot o afo
        suonato = false;		
		if(spot > 0){
           cspot++;
		   if(cspot == spot){
		       Player.advertise(random(nspot));
			   startplay=millis();
			   suonato = true;
			   cspot =0;
		   }	   
	    }
		if(seg > 0 && !suonato){
           cseg++;
		   if(cseg == seg){
			   Player.playFolder(8,random(fsize[8]+1));
			   suonato= true;
			   startplay=millis();
			   cseg =0;
		   }	   
	    }
		if(afo > 0 && !suonato > 0){
           cafo++;
		   if(cafo == afo){
			   Player.advertise(51+random(nafo));
			   startplay=millis();
			   suonato = true;;
			   cseg =0;
		   }	   
	    }


		if (!suonato){	
		    if(aradio) {
		    // modalità random attiva 
               punt = scegli(maxpunt);
               suonato = true;			   
            }
            if(cont) {
		       // modalità continua	
		           punt++;
               if(punt >maxpunt)
                  punt =1;			   
		    }
		    if(suonato) {
				suona(punt);
			    startplay = millis();
			}	
		}
    }
 

// regola volume in base alla sintonia:
// il volume si abbassa fino ad azzararsi nel punto  
// centrale tra una 'stazione' e la successiva 

if(sintonizzo) {
 	 volusint = 100 - (deltas * 100)/deltamax;
	 volusint = volusint * volump3/100;
   if(volusint != volumep) {
     volumep = volusint;
     Player.volume(volusint);
     if (debug) Serial.print("volume sintonia =");
     if (debug) Serial.println(volusint);
	 // se ci si sta muovendo si suona l'audio della sintonia ogni 3 sec.
	 if(millis()-Tsin > Isin) {
	     Player.playFolder(10,random(fsize[10]+1));
	     Tsin=millis();
	 }	 
  }
} 






if(debug && random(33) == 0) {

     
      Serial.print(" mode= ");   
      Serial.print(mode);
      Serial.print(" sinto= ");   
      Serial.print(sinto);
      Serial.print(" s.nizzo= ");   
      Serial.print(sintonizzo);
      Serial.print(" radio= ");
      Serial.print(aradio);
      Serial.print(" volump3= ");
      Serial.print(volump3);     
      Serial.print(" volusint= ");
      Serial.print(volusint);     
      Serial.print(" punt= ");  
      Serial.print(punt);
      Serial.print(" deltas= ");   
      Serial.print(deltas);
      Serial.print(" maxpunt= ");  
      Serial.println(maxpunt);  
      Serial.print(" Folders:  "); 
      for(f=1;f<10;f++) {
      Serial.print(fold[f]);
      Serial.print(" ");
      }
      Serial.println(" ");    
}

  
} 

// ------------------------- fine ciclo generale temporizzato ---------------------

// operazioni che coinvolgono la sintonia

// calcola deltas e deltamax  :  

sinto = analogRead(pinsint);  // legge posizione sintonia
rmode = dovesono(sinto);
deltas = abs(sintpoint[rmode] - sinto);
if(sinto >= sintpoint[rmode])
	 deltamax = abs((sintpoint[rmode+1]-sintpoint[rmode])/2);
else
	 deltamax = abs((sintpoint[rmode-1]-sintpoint[rmode])/2);


// led sintonia 

if(deltas <= istsint){
    analogWrite(pin_LED,255);     
}
else
   analogWrite(pin_LED,0); 

// segnale sonoro 
// suona solo se siamo fuori sintonia
if(deltas > istsint) {
  delayMicroseconds(map(deltas, 0, deltamax, 500, 10000));
  if(segnale) {
    segnale = false;
      digitalWrite(pin_AUDIO,LOW);  
  }
  else {
    segnale = true;
      digitalWrite(pin_AUDIO,HIGH);
  }
  sintonizzo = true;
  Ts = millis();
} 



// decide di cambiare canale quando la sintonia è stabile per oltre Is millisecondi
if(millis()-Ts > Is && sintonizzo) {
  sintonizzo = false;
    k = dovesono(sinto);
    if (k == 0)
    initscala=true;
  if (k == 6)
    fondoscala=true;
    if (initscala && fondoscala) {  // una escursione inizio e fondo scala abilita il flag segreto
        segreto = true;   
        leggiprog(mode);
    initscala = false;
    fondoscala = false;
  }
  if (k >0 && k <6) {
    mode = k;
    leggiprog(mode);
  } 
}


// regolazione volume 
// durante la sintonia il volume viene ignorato 
if(!sintonizzo) {

   volume = analogRead(pinvol);

   if(abs(volume-volumep) > istvol){
       volump3 = map(volume, 0, 1023, 0, 30);
       if(debug) Serial.print("Modifico volume:");
       if(debug) Serial.println(volump3);
       volumep = volume;
       Player.volume(volump3);
   }

}


 

}  // fine loop principale 
 




// funzione dovesono , individua il setpoint della sintonia  in base al
// valore del potenziometro con s che può variare da 0 a 1023 
byte dovesono(int s) {
int p =0;
int pr =0;
int j = 0;
int i =0;
p = (sintpoint[6]-sintpoint[5])/2;
// inizio e fine scala
  if(s < sintpoint[1]/2) 
      return 0;
  if(s > sintpoint[5]+p)
      return 6;	
   pr= sintpoint[1]/2;
   for (i = 1;i < 6;i++) {
      j = sintpoint[i];
	  p = j+(sintpoint[i+1]-sintpoint[i])/2;
      if(s >= pr && s <= p)
		  break; 
     pr = p;
  }
  return i;
}



// funzione suona
// individua il pezzo 'punt' lo relativizza e lo suona
void suona(int p) {
int j;
for(j=1;j<=nfold;j++) {
  if(fold[j] > 0) {
	   if (p > fsize[j])
         p = p - fsize[j];
    else {
        Player.playFolder(j, p); 
		foldcor = j;
		puntcor = p;
		if(debug) {
		   	Serial.print("Suono F:");  
		    Serial.print(j);  
		   	Serial.print(" N:");  
		    Serial.println(p);
		}	
		break;
    }  	   
  }
}	  

}



// funzione scegli 
// sceglie il puntatore in base alle % di probabilità delle singole cartelle
int scegli(int maxp) {
int j,jj;
int pnt;
bool fatto = true;
while(fatto){
  pnt = random(maxp)+1;
  for(j=1;j<=nfold;j++) {
    if(fold[j] > 0) {
	    if (pnt > fsize[j])
            pnt = pnt - fsize[j];
        else {
            if(random(100/fold[j]) == 0)  
                fatto = false;
		    break;
       }  	   
    }
  }	  
}
  // recupera pnt assoluto

  for(jj=1;jj<=j-1;jj++) {

      if(fold[jj]>0)
      pnt = pnt + fsize[jj];
   }
return pnt;

}




// funzione absuona
// individua il pezzo 'punt' assoluto lo relativizza e lo suona
void absuona(int p) {
int j;
for(j=1;j<=nfold;j++) {
   if (p > fsize[j])
         p = p - fsize[j];
    else {
        Player.playFolder(j, p); 
        if(debug) {
        Serial.print("Suono abs F:");  
        Serial.print(j);  
        Serial.print(" N:");  
        Serial.println(p);
    } 
    	break;
    }  	   
  }
}	  


// funzione leggiprog
// inizializza variabili radio spot afo e abilitazione folders
void leggiprog(byte p){
int i = 1;
int j;
byte f,c,n;
if(segreto) 
	riempiseg(p);
else
	riempi(p);
// azzera fold()
for(j=1;j<=nfold;j++){
  fold[j]=0;
  }
aradio = false;
spot = 0;
afo = 0; 
seg = 0;
cont = false;
while (prog[i][0] > 0)
	{
   
    // legge comando 
	c = prog[i][0];
	f = prog[i][1];
	n = prog[i][2];
	switch(c) {
		case 1 :
		   // abilita folder
		   fold[f] = n;
           break;	
		case 2 :
		   // spot
		   spot = f;
           break;	
		case 3 :
		   // afo
		   afo = f;
           break;	
		case 4 :
		   // seg
		   seg = f;
           break;	
		case 5 :
		   // continuo
		   cont = true;
           break;	
		case 6 :
		   // abilita da folder a folder (la % prob è fissa al 100%)
		   for(j = f;j <= n;j++) {
 		     fold[j] = 100;
       }
	       break;	
		case 7 :
		   // abilita radio
		   aradio = true;
	       break;
        default:
		    // range con % 
	    	for(j = f;j <= n;j++) {
 		      fold[j] = c - 10;
            }
            break;					   
    }
	i++;
  }
  // calcola maxpunt 
  maxpunt=0;
  for(i=1;i<=nfold;i++) {
	  if(fold[i] > 0)
		  maxpunt = maxpunt + fsize[i];
  }	  
}


	
void riempi(byte m) {
int i;
// in base a modo riempie il vettore programma (prog1 e 2)
// ma prima lo azzera 
 for(i=0;i<=10;i++){
	prog[i][0]=0;
	prog[i][1]=0;
	prog[i][2]=0;
 }	

/* -------- Tabella comandi ---------
BYTE 1 :  [nn][0]
1 = Folder N <0-100%>
2 = Spot N
3 = Afo N
4 = seg N
5 = continuo 
6 = frange N1 N2
7 = radio (randomize)
10-110 = frange N1 N2

BYTE 2 :  [nn][1]
numero folder, N, N1 

BYTE 3 :  [nn][2]
N2, <0-100> (%folder) 
------------------------------------ */ 

 switch(m) {
   case 1 :
     // Default:  canzoni scaricate, programmi originali + memory 30% + spot(5)  
	prog[1][0]=2;
    prog[1][1]=5;
	prog[2][0]=1;
	prog[2][1]=1;
	prog[2][2]=100;
	prog[3][0]=1;
	prog[3][1]=3;
	prog[3][2]=100;
	prog[4][0]=1;
	prog[4][1]=4;
	prog[4][2]=30;
	prog[5][0]=7;
    break;
   case 2 :
     // solo contenuto d'epoca registrato + spot(5)
	prog[1][0]=2;
    prog[1][1]=5;
	prog[2][0]=6;
	prog[2][1]=2;
	prog[2][2]=4;
	prog[3][0]=7;
    break;
   case 3 :
     //  solo canzoni scaricate + memory 40%
	prog[1][0]=1;
	prog[1][1]=1;
	prog[1][2]=100;
	prog[2][0]=1;
	prog[2][1]=4;
	prog[2][2]=40;		
	prog[3][0]=7;
    break;
   case 4 :
     // solo canzoni scaricate + memory 20% con intrusioni repertorio al 50%
	prog[1][0]=1;
	prog[1][1]=1;
	prog[1][2]=100;
	prog[2][0]=1;
	prog[2][1]=5;
	prog[2][2]=50;
	prog[3][0]=1;
	prog[3][1]=4;
	prog[3][2]=20;	
	prog[4][0]=7;
    break;
   case 5 : 
    // solo musica classica  + favole 50%
	prog[1][0]=1;
	prog[1][1]=9;
	prog[1][2]=100;
	prog[2][0]=1;
	prog[2][1]=6;
	prog[2][2]=50;
	prog[3][0]=7;
    break;
 }	
}	


void riempiseg(byte m) {
int i;
// in base a modo riempie il vettore programma (prog1 e 2)
// ma prima lo azzera 
 for(i=0;i<=10;i++){
	prog[i][0]=0;
	prog[i][1]=0;	
    prog[i][2]=0;  
 }	

/* -------- Tabella comandi ---------
BYTE 1 :  [nn][0]
1 = Folder N <0-100%>
2 = Spot N
3 = Afo N
4 = seg N
5 = continuo 
6 = frange N1 N2
7 = radio (randomize)
10-110 = frange N1 N2

BYTE 2 :  [nn][1]
numero folder, N, N1 

BYTE 3 :  [nn][2]
N2, <0-100> (%folder) 
------------------------------------ */ 
 
 switch(m) {
    case 1 :
     // repertorio + canzoni scaricate + Afo seg + Afo  
	prog[1][0]=3;
  prog[1][1]=2;
	prog[2][0]=4;
  prog[2][1]=5;
	prog[3][0]=1;
	prog[3][1]=1;
	prog[3][2]=100;
	prog[4][0]=1;
	prog[4][1]=5;
	prog[4][2]=100;
	prog[5][0]=7;
    break;
   case 2 :
     // tutto il contenuto + spot + afo
	prog[1][0]=2;
    prog[1][1]=3;
	prog[2][0]=3;
    prog[2][1]=5;
	prog[3][0]=6;
	prog[3][1]=1;
	prog[3][2]=10;
	prog[4][0]=7;
    break;
   case 3 :
     //  solo favole + afo
	prog[1][0]=2;
	prog[1][1]=3;
	prog[2][2]=1;
	prog[2][0]=6;
	prog[2][1]=100;
	prog[3][0]=7;
    break;
   case 4 :
     // solo contenuto live
	prog[1][0]=1;
	prog[1][1]=4;
	prog[1][2]=100;
	prog[4][0]=7;
    break;
   case 5 : 
    // solo musica classica al 20% + afo + seg + letture seg
	prog[1][0]=1;
	prog[1][1]=9;
	prog[1][2]=20;
	prog[2][0]=6;
	prog[2][0]=7;
	prog[2][0]=8;
	prog[5][0]=7;
    break;
 }	
}	
